
# q-expansion of newform 40.15.e.b, downloaded from the LMFDB on 15 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 40
weight = 15
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[31, [-1]], [21, [-1]], [17, [-1]]]
aps_data = [[128], [0], [-78125], [67866], [16052382], [-125276586], [0], [644127118], [5951338874], [0], [0], [100482630246], [227781371922], [0], [985929843946], [-1801585778906], [4687670223678], [0], [0], [0], [0], [0], [0], [-57601022824782], [0], [0], [-245852697774246], [0], [0], [0], [202034503450506], [-943716185171538], [0], [1734921854593118], [0], [0], [-4526399772425514], [0], [-5509085207457574], [3277577949137014], [-11543559819333042], [0], [0], [0], [21742592723136326], [0], [-31967202364821298], [41192709319899594], [0], [0], [0], [0], [67993552791178402], [-79568429656170498], [0], [147099966223518554], [0], [0], [-35608562372038074], [228956478982926162], [0], [-96499501773150106], [0], [0], [0], [-503679305723087434], [-498299326931100578], [0], [0], [0], [0], [0], [-1371191763441060054], [1583911652270049414], [1494690495960196478], [446539020163625674], [0], [-11411480405223594], [-707197984136104638], [-894470937414819022], [4435810800023388078], [0], [0], [0], [0], [0], [-6563111831218225662], [0], [0], [1854417147947088234], [0], [0], [560133057303698586], [-11101553203419042978], [4774688727956193358], [-16181830247397549446], [0], [-7518420993616084878], [0], [0], [0], [-20852430243551161994], [0], [-31537751897655624462], [-25313897293317819458], [0], [0], [0], [0], [14784085706019346642], [-42128768214773066934], [2084546188666804614], [0], [48206916907626401438], [0], [-75667942503316801278], [0], [92796678475689337306], [34165469752379252054], [-43996927618138601202], [0], [0], [8038581588081140486], [0], [-139169962562930273138], [0], [0], [0], [85259169361827144186], [-220447576402960122666], [-14787962915530698002], [147710700252182355994], [0], [-165156851537017257594], [-253390691282384380398], [-317053562669019669662], [291108306043276829894], [0], [46547510356102169206], [-424140678063150791502], [-406829145501216221218], [0], [-114448951798999466886], [0], [0], [0], [-138829190662485873306], [0], [671279322965804606078], [315393224916351631754], [718319318935745067126], [-816331210565519112318], [0], [428302848342625276346], [0], [0], [0], [497319088183661277378], [0], [0], [0], [0], [715287289463618069466], [239361842186176449822], [0], [-1676703466012472567366], [0], [1954405042550408745606]]
